package cd.casic.ci.common.pipeline.pojo.cascade;

import cd.casic.ci.common.pipeline.enums.BuildFormPropertyType;
import cd.casic.ci.common.pipeline.pojo.BuildFormProperty;
import cd.casic.ci.common.pipeline.utils.CascadePropertyUtils;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.pojo.cascade
 * @Project：ops-pro
 * @name：CascadeParam
 * @Date：2025/03/26 10:05
 * @Filename：CascadeParam
 * @description：Todo
 */

@Data
@Slf4j
@AllArgsConstructor
public abstract class CascadeParam {
    private final BuildFormPropertyType type;
    private final List<String> chain;

    public BuildCascadeProps getProps(BuildFormProperty prop, String projectId) {
        if (chain.size() < 2 || chain.size() != chainHandler().size()) {
            // 最少两个链路节点，且节点数和链式处理器数量相等
            throw new IllegalArgumentException("chain size must be 2 and equal to chainHandler size");
        }
        Map<String, String> defaultValue = getDefaultValue(prop);
        Map<String, BuildCascadeProps> map = new HashMap<>();
        for (String key : chain) {
            CascadeParamPropsHandler propsHandler = chainHandler().get(key);
            if (propsHandler == null) {
                throw new IllegalArgumentException("can not find handler for " + key + "|" + type);
            }
            map.put(key, propsHandler.handle(key, defaultValue.getOrDefault(key, ""), projectId));
        }
        // 链式关系处理
        for (int i = chain.size() - 1; i > 0; i--) {
            map.get(chain.get(i - 1)).setChildren(map.get(chain.get(i)));
        }
        return map.get(chain.get(0));
    }

    private Map<String, String> getDefaultValue(BuildFormProperty prop) {
        Map<String, String> defaultValue = CascadePropertyUtils.parseDefaultValue(prop.getId(), prop.getDefaultValue(), prop.getType());
        return chain.stream().allMatch(defaultValue::containsKey) ? defaultValue : Collections.emptyMap();
    }

    public abstract Map<String, CascadeParamPropsHandler> chainHandler();

    public interface CascadeParamPropsHandler {
        BuildCascadeProps handle(String key, String defaultValue, String projectId);
    }

    private static final Logger logger = LoggerFactory.getLogger(CascadeParam.class);
}