package cd.casic.ci.common.pipeline.pojo.element.agent;

import cd.casic.ci.common.pipeline.pojo.element.Element;
import cd.casic.ci.common.pipeline.pojo.element.atom.ManualReviewParam;
import cd.casic.ci.common.pipeline.pojo.transfer.PreStep;
import cd.casic.ci.common.pipeline.utils.TransferUtil;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.pojo.element.agent
 * @Project：ops-pro
 * @name：ManualReviewUserTaskElement
 * @Date：2025/03/26 16:44
 * @Filename：ManualReviewUserTaskElement
 * @description：Todo
 */
@Data
@NoArgsConstructor
@Schema(title = "人工审核", description = ManualReviewUserTaskElement.classType)
public class ManualReviewUserTaskElement extends Element {
    public static final String classType = "manualReviewUserTask";

    @Schema(title = "任务名称", required = true)
    private String name = "人工审核";
    @Schema(title = "id", required = false)
    private String id;
    @Schema(title = "状态", required = false)
    private String status;
    @Schema(title = "审核人", required = true)
    private List<String> reviewUsers;
    @Schema(title = "描述", required = false)
    private String desc;
    @Schema(title = "审核意见", required = false)
    private String suggest;
    @Schema(title = "参数列表", required = false)
    private List<ManualReviewParam> params;
    @Schema(title = "输出变量名空间", required = false)
    private String namespace;
    @Schema(title = "发送的通知类型", required = false)
    private List<String> notifyType;
    @Schema(title = "发送通知的标题", required = false)
    private String notifyTitle;
    @Schema(title = "是否以markdown格式发送审核说明", required = false)
    private Boolean markdownContent;
    @Schema(title = "企业微信群id", required = false)
    private List<String> notifyGroup;
    @Schema(title = "审核提醒时间（小时），支持每隔x小时提醒一次", required = false)
    private Integer reminderTime;

    @Override
    public String getTaskAtom() {
        return "manualReviewTaskAtom";
    }

    public PreStep transferYaml(JSONObject defaultValue) {
        Map<String, Object> input = new HashMap<>();
        if (reviewUsers != null && !reviewUsers.isEmpty()) {
            input.put("reviewUsers", reviewUsers);
        }
        if (desc != null && !desc.isEmpty()) {
            input.put("desc", desc);
        }
        if (suggest != null && !suggest.isEmpty()) {
            input.put("suggest", suggest);
        }
        if (params != null && !params.isEmpty()) {
            input.put("params", params);
        }
        if (namespace != null && !namespace.isEmpty()) {
            input.put("namespace", namespace);
        }
        if (notifyType != null && !notifyType.isEmpty()) {
            input.put("notifyType", notifyType);
        }
        if (notifyTitle != null && !notifyTitle.isEmpty()) {
            input.put("notifyTitle", notifyTitle);
        }
        if (markdownContent != null) {
            input.put("markdownContent", markdownContent);
        }
        if (notifyGroup != null && !notifyGroup.isEmpty()) {
            input.put("notifyGroup", notifyGroup);
        }
        if (reminderTime != null) {
            input.put("reminderTime", reminderTime);
        }
        return new PreStep(
                name,
                id,
                getAtomCode() + "@" + getVersion(),
                TransferUtil.simplifyParams(defaultValue, input)
        );
    }

    @Override
    public String getClassType() {
        return classType;
    }
}
