package cd.casic.ci.common.pipeline.pojo.element.trigger;

import cd.casic.ci.common.pipeline.enums.StartType;
import cd.casic.ci.common.pipeline.enums.TriggerRepositoryType;
import cd.casic.ci.common.pipeline.pojo.element.ElementProp;
import cd.casic.ci.common.pipeline.pojo.element.trigger.enums.CodeEventType;
import cd.casic.ci.common.pipeline.utils.TriggerElementPropUtils;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.List;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.pojo.element.trigger
 * @Project：ops-pro
 * @name：CodeP4WebHookTriggerElement
 * @Date：2025/03/27 10:20
 * @Filename：CodeP4WebHookTriggerElement
 * @description：Todo
 */
@Accessors(chain = true)
@Schema(title = "p4事件触发", description = CodeP4WebHookTriggerElement.classType)
public class CodeP4WebHookTriggerElement extends WebHookTriggerElement {
    public static final String classType = "codeP4WebHookTrigger";

    @Schema(title = "任务名称", required = true)
    private String name = "P4事件触发";

    @Schema(title = "id", required = false)
    private String id;

    @Schema(title = "状态", required = false)
    private String status;

    @Schema(title = "数据", required = true)
    private CodeP4WebHookTriggerData data;

    public CodeP4WebHookTriggerElement(String name, String id, String status) {
        super(name, id, status);
    }

    @Override
    public String getClassType() {
        return classType;
    }

    @Override
    public String findFirstTaskIdByStartType(StartType startType) {
        return StartType.WEB_HOOK.name().equals(startType.name()) ? this.id : super.findFirstTaskIdByStartType(startType);
    }

    @Override
    public List<ElementProp> triggerCondition() {
        CodeP4WebHookTriggerInput input = data.getInput();
        List<ElementProp> props;
        switch (input.getEventType()) {
            case CHANGE_COMMIT:
            case CHANGE_SUBMIT:
            case CHANGE_CONTENT:
            case SHELVE_COMMIT:
            case SHELVE_SUBMIT:
            case SHELVE_DELETE:
                props = List.of(
                        TriggerElementPropUtils.vuexInput("includePaths", input.getIncludePaths()),
                        TriggerElementPropUtils.vuexInput("excludePaths", input.getExcludePaths())
                );
                break;
            default:
                props = List.of();
        }
        return props.stream().filter(java.util.Objects::nonNull).toList();
    }
}

@Data
@Accessors(chain = true)
class CodeP4WebHookTriggerData {
    private CodeP4WebHookTriggerInput input;
}

@Data
@Accessors(chain = true)
class CodeP4WebHookTriggerInput {
    @Schema(title = "仓库ID", required = true)
    private String repositoryHashId;

    @Schema(title = "新版的git原子的类型")
    private TriggerRepositoryType repositoryType;

    @Schema(title = "新版的git代码库名")
    private String repositoryName;

    @Schema(title = "用于包含的路径", required = false)
    private String includePaths;

    @Schema(title = "用于排除的路径", required = false)
    private String excludePaths;

    @Schema(title = "事件类型", required = false)
    private CodeEventType eventType;
}
