package cd.casic.ci.common.pipeline.type.docker;

import cd.casic.ci.common.pipeline.type.BuildType;
import cd.casic.ci.common.pipeline.type.DispatchRouteKeySuffix;
import cd.casic.ci.common.pipeline.type.StoreDispatchType;
import cd.casic.ci.common.pipeline.utils.EnvUtils;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;

import java.util.Map;

/**
 * @author by mianbin
 * @Classname DockerDispatchType
 * @Description TODO
 * @Date 2025/3/31 9:44
 */
@Getter
@Setter
@EqualsAndHashCode(callSuper = true)
public class DockerDispatchType extends StoreDispatchType {
    @JsonProperty("value")
    @Schema(title = "docker构建版本", required = false)
    private String dockerBuildVersion;

    @Schema(title = "镜像类型", required = false)
    private ImageType imageType = ImageType.BKDEVOPS;

    @Schema(title = "凭证id", required = false)
    private String credentialId = "";

    @Schema(title = "凭证项目id", required = false)
    private String credentialProject = "";

    @Schema(title = "商店镜像代码", required = false)
    private String imageCode = "";

    @Schema(title = "商店镜像版本", required = false)
    private String imageVersion = "";

    @Schema(title = "商店镜像名称", required = false)
    private String imageName = "";

    @Schema(title = "docker资源配置ID", required = false)
    private int performanceConfigId = 0;

    @Schema(title = "镜像仓库用户名", required = false)
    private String imageRepositoryUserName = "";

    @Schema(title = "镜像仓库密码", required = false)
    private String imageRepositoryPassword = "";

    public DockerDispatchType(String dockerBuildVersion, ImageType imageType, String credentialId, String credentialProject, String imageCode, String imageVersion, String imageName, int performanceConfigId, String imageRepositoryUserName, String imageRepositoryPassword) {
        super(
                dockerBuildVersion == null || dockerBuildVersion.isBlank() ? imageCode : dockerBuildVersion,
                DispatchRouteKeySuffix.DOCKER_VM,
                imageType,
                credentialId,
                credentialProject,
                imageCode,
                imageVersion,
                imageName
        );
        this.dockerBuildVersion = dockerBuildVersion;
        this.imageType = imageType;
        this.credentialId = credentialId;
        this.credentialProject = credentialProject;
        this.imageCode = imageCode;
        this.imageVersion = imageVersion;
        this.imageName = imageName;
        this.performanceConfigId = performanceConfigId;
        this.imageRepositoryUserName = imageRepositoryUserName;
        this.imageRepositoryPassword = imageRepositoryPassword;
    }

    @Override
    public void cleanDataBeforeSave() {
        this.dockerBuildVersion = this.dockerBuildVersion != null ? this.dockerBuildVersion.trim() : null;
        this.credentialId = this.credentialId != null ? this.credentialId.trim() : null;
        this.credentialProject = this.credentialProject != null ? this.credentialProject.trim() : null;
        this.imageCode = this.imageCode != null ? this.imageCode.trim() : null;
        this.imageVersion = this.imageVersion != null ? this.imageVersion.trim() : null;
        this.imageName = this.imageName != null ? this.imageName.trim() : null;
    }

    @Override
    public BuildType buildType() {
        return BuildType.valueOf(BuildType.DOCKER.name());
    }

    @Override
    public void replaceField(Map<String, String> variables) {
        this.dockerBuildVersion = EnvUtils.parseEnv(this.dockerBuildVersion, variables);
        this.credentialId = EnvUtils.parseEnv(this.credentialId, variables);
    }
}
