package cd.casic.ci.common.pipeline.type.kubernetes;

/**
 * @author by mianbin
 * @Classname KubernetesDispatchType
 * @Description TODO
 * @Date 2025/3/31 9:41
 */

import cd.casic.ci.common.pipeline.type.BuildType;
import cd.casic.ci.common.pipeline.type.DispatchRouteKeySuffix;
import cd.casic.ci.common.pipeline.type.StoreDispatchType;
import cd.casic.ci.common.pipeline.type.docker.ImageType;
import cd.casic.ci.common.pipeline.utils.EnvUtils;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.EqualsAndHashCode;

import java.util.Map;


@EqualsAndHashCode(callSuper = true)
public class KubernetesDispatchType extends StoreDispatchType {

    /**
     * Kubernetes构建版本
     */
    @JsonProperty("value")
    private String kubernetesBuildVersion;

    /**
     * 镜像类型，默认为BKDEVOPS
     */
    private ImageType imageType = ImageType.BKDEVOPS;

    /**
     * 凭证ID
     */
    private String credentialId = "";

    /**
     * 凭证所属项目
     */
    private String credentialProject = "";

    /**
     * 商店镜像代码
     */
    private String imageCode = "";

    /**
     * 商店镜像版本
     */
    private String imageVersion = "";

    /**
     * 商店镜像名称
     */
    private String imageName = "";

    /**
     * Docker资源配置ID，默认为0
     */
    private int performanceConfigId = 0;

    /**
     * 构造函数，初始化Kubernetes调度类型的相关参数
     * @param kubernetesBuildVersion Kubernetes构建版本
     * @param imageType 镜像类型
     * @param credentialId 凭证ID
     * @param credentialProject 凭证所属项目
     * @param imageCode 商店镜像代码
     * @param imageVersion 商店镜像版本
     * @param imageName 商店镜像名称
     * @param performanceConfigId Docker资源配置ID
     */
    public KubernetesDispatchType(String kubernetesBuildVersion, ImageType imageType, String credentialId,
                                  String credentialProject, String imageCode, String imageVersion, String imageName,
                                  int performanceConfigId) {
        super(
                kubernetesBuildVersion == null || kubernetesBuildVersion.isEmpty() ? imageCode : kubernetesBuildVersion,
                DispatchRouteKeySuffix.KUBERNETES,
                imageType,
                credentialId,
                credentialProject,
                imageCode,
                imageVersion,
                imageName
        );
        this.kubernetesBuildVersion = kubernetesBuildVersion;
        this.imageType = imageType;
        this.credentialId = credentialId;
        this.credentialProject = credentialProject;
        this.imageCode = imageCode;
        this.imageVersion = imageVersion;
        this.imageName = imageName;
        this.performanceConfigId = performanceConfigId;
    }

    /**
     * 在保存数据之前清理数据，去除字符串字段的首尾空格
     */
    @Override
    public void cleanDataBeforeSave() {
        this.kubernetesBuildVersion = this.kubernetesBuildVersion != null ? this.kubernetesBuildVersion.trim() : null;
        this.credentialId = this.credentialId != null ? this.credentialId.trim() : null;
        this.credentialProject = this.credentialProject != null ? this.credentialProject.trim() : null;
        this.imageCode = this.imageCode != null ? this.imageCode.trim() : null;
        this.imageVersion = this.imageVersion != null ? this.imageVersion.trim() : null;
        this.imageName = this.imageName != null ? this.imageName.trim() : null;
    }

    /**
     * 获取构建类型，返回Kubernetes类型
     * @return 构建类型
     */
    @Override
    public BuildType buildType() {
        return BuildType.valueOf(BuildType.KUBERNETES.name());
    }

    /**
     * 根据变量映射替换字段值
     * @param variables 变量映射
     */
    @Override
    public void replaceField(Map<String, String> variables) {
        this.kubernetesBuildVersion = EnvUtils.parseEnv(this.kubernetesBuildVersion, variables);
        this.credentialId = EnvUtils.parseEnv(this.credentialId, variables);
    }
}
