package cd.casic.pipeline.dispatch.api;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.dispatch.pojo.JobQuotaProject;
import cd.casic.pipeline.dispatch.pojo.enums.JobQuotaVmType;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@Tag(name = "OP_JOBS_PROJECT_QUOTA", description = "Job配额管理")
@RestController
@RequestMapping("/op/jobs/quota")
public interface OpJobQuotaProjectResource {

    @Operation(summary = "获取全部的JOB配额信息")
    @GetMapping("/list/project/quota")
    CommonResult<List<JobQuotaProject>> list(
            @Parameter(description = "项目ID", required = false)
            @RequestParam("projectId") String projectId
    );

    @Operation(summary = "获取项目的JOB配额信息")
    @GetMapping("/{projectId}/{vmType}")
    CommonResult<JobQuotaProject> get(
            @Parameter(description = "项目ID", required = true)
            @PathVariable("projectId") String projectId,
            @Parameter(description = "构建机类型", required = true)
            @PathVariable("vmType") JobQuotaVmType vmType,
            @Parameter(description = "构建来源", required = false)
            @RequestParam("channelCode") String channelCode
    );

    @Operation(summary = "添加项目的JOB配额信息")
    @PutMapping("/{projectId}")
    CommonResult<Boolean> add(
            @Parameter(description = "项目ID", required = true)
            @PathVariable("projectId") String projectId,
            @Parameter(description = "Job配额信息", required = true)
            JobQuotaProject jobQuota
    );

    @Operation(summary = "删除项目的JOB配额信息")
    @DeleteMapping("/{projectId}/{vmType}")
    CommonResult<Boolean> delete(
            @Parameter(description = "项目ID", required = true)
            @PathVariable("projectId") String projectId,
            @Parameter(description = "构建机类型", required = true)
            @PathVariable("vmType") JobQuotaVmType vmType,
            @Parameter(description = "构建来源", required = false)
            @RequestParam("channelCode") String channelCode
    );

    @Operation(summary = "更新项目的JOB配额信息")
    @PostMapping("/{projectId}/{vmType}")
    CommonResult<Boolean> update(
            @Parameter(description = "项目ID", required = true)
            @PathVariable("projectId") String projectId,
            @Parameter(description = "构建机类型", required = true)
            @PathVariable("vmType") JobQuotaVmType vmType,
            @Parameter(description = "Job配额信息", required = true)
            JobQuotaProject jobQuota
    );

    @Operation(summary = "清零异常的构建配额记录")
    @PostMapping("/project/{projectId}/vm/{vmType}")
    CommonResult<Boolean> restoreProjectRunningJobs(
            @Parameter(description = "项目ID", required = true)
            @PathVariable("projectId") String projectId,
            @Parameter(description = "构建机类型", required = true)
            @PathVariable("vmType") JobQuotaVmType vmType,
            @Parameter(description = "构建时间", required = true)
            @RequestParam("createTime") String createTime,
            @Parameter(description = "构建来源", required = false)
            @RequestParam("channelCode") String channelCode
    );
}