package cd.casic.ci.log.api;

import cd.casic.ci.log.dal.pojo.TaskBuildLogProperty;
import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.ci.log.dal.pojo.message.LogMessage;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;

import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.*;;

/**
 * @author by mianbin
 * @Classname BuildLogPrintResource
 * @Description
 * @Date 2025/3/19 14:44
 */
@Tag(name = "构建-日志资源", description = "BUILD_LOG_PRINT")
@RestController
@RequestMapping("/build/logs")
public interface BuildLogPrintResource {

    @Operation(description = "写入一条日志")
    @PostMapping("/")
    CommonResult<Boolean> addLogLine(
            @Parameter(description = "构建ID", required = true)
            @RequestHeader(AUTH_HEADER_DEVOPS_BUILD_ID)
            String buildId,
            @Parameter(description = "一条日志", required = true)
            LogMessage logMessage);

    @Operation(description = "写入一条红色高亮日志")
    @PostMapping("/red")
    CommonResult<Boolean> addRedLogLine(
            @Parameter(description = "构建ID", required = true)
            @RequestHeader(AUTH_HEADER_DEVOPS_BUILD_ID)
            String buildId,
            @Parameter(description = "一条日志", required = true)
            LogMessage logMessage);

    @Operation(description = "写入一条黄色高亮日志")
    @PostMapping("/yellow")
    CommonResult<Boolean> addYellowLogLine(
            @Parameter(description = "构建ID", required = true)
            @RequestHeader(AUTH_HEADER_DEVOPS_BUILD_ID)
            String buildId,
            @Parameter(description = "一条日志", required = true)
            LogMessage logMessage
    );

    @Operation(description = "写入多条日志")
    @PostMapping("/multi")
    CommonResult<Boolean> addMultiLine(
            @Parameter(description = "构建ID", required = true)
            @RequestHeader(AUTH_HEADER_DEVOPS_BUILD_ID)
            String buildId,
            @Parameter(description = "多条日志列表", required = true)
            List<LogMessage> logMessages
    );

    @Operation(description = "写入日志状态")
    @PostMapping("/status")
    CommonResult<Boolean> addLogStatus(
            @Parameter(description = "构建id", required = true)
            @RequestHeader(AUTH_HEADER_DEVOPS_BUILD_ID)
            String buildId,
            @Parameter(description = "分辨插件的tag，默认填对应插件id")
            @RequestParam("tag")
            String tag,
            @Parameter(description = "插件内的分类的子tag，默认为空")
            @RequestParam("subTag")
            String subTag,
            @Parameter(description = "job id或者container的34位id", required = false)
            @RequestParam("jobId")
            String jobId,
            @Parameter(description = "执行次数", required = false)
            @RequestParam("executeCount")
            int executeCount,
            @Parameter(description = "日志存储模式", required = false)
            @RequestParam("logMode")
            String logMode
    );

    @Operation(description = "更新日志状态")
    @PutMapping("/status")
    CommonResult<Boolean> updateLogStatus(
            @Parameter(description = "构建id", required = true)
            @RequestHeader(AUTH_HEADER_DEVOPS_BUILD_ID)
            String buildId,
            @Parameter(description = "是否已构建完成", required = true)
            @RequestParam("finished")
            boolean finished,
            @Parameter(description = "分辨插件的tag，默认填对应插件id")
            @RequestParam("tag")
            String tag,
            @Parameter(description = "插件内的分类的子tag，默认为空")
            @RequestParam("subTag")
            String subTag,
            @Parameter(description = "job id或者container的34位id", required = false)
            @RequestParam("jobId")
            String jobId,
            @Parameter(description = "执行次数", required = false)
            @RequestParam("executeCount")
            int executeCount,
            @Parameter(description = "日志存储模式", required = false)
            @RequestParam("logMode")
            String logMode
    );

    @Operation(description = "更新日志存储模式的流转状态")
    @PostMapping("/mode")
    CommonResult<Boolean> updateLogStorageMode(
            @Parameter(description = "构建ID", required = true)
            @RequestHeader(AUTH_HEADER_DEVOPS_BUILD_ID)
            String buildId,
            @Parameter(description = "执行次数")
            @RequestParam("executeCount")
            int executeCount,
            @Parameter(description = "所有插件的日志存储结果", required = true)
            List<TaskBuildLogProperty> propertyList
    );

}
