package cd.casic.ci.process.api.process.pojo.pipeline.record;

import cd.casic.ci.common.pipeline.container.Container;
import cd.casic.ci.common.pipeline.container.TriggerContainer;
import cd.casic.ci.common.pipeline.container.VMBuildContainer;
import cd.casic.ci.common.pipeline.enums.BuildRecordTimeStamp;
import cd.casic.ci.common.pipeline.enums.BuildStatus;
import cd.casic.ci.common.pipeline.pojo.element.Element;
import cd.casic.ci.common.pipeline.pojo.time.BuildTimestampType;
import cd.casic.ci.common.pipeline.utils.ElementUtils;
import cd.casic.ci.process.api.process.pojo.app.StartBuildContext;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author by mianbin
 * @Classname BuildRecordContainer
 * @Description TODO
 * @Date 2025/3/31 17:12
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Schema(title = "构建详情记录-插件任务")
public class BuildRecordContainer {
    @Schema(title = "构建ID", required = true)
    private String buildId;
    @Schema(title = "项目ID", required = true)
    private String projectId;
    @Schema(title = "流水线ID", required = true)
    private String pipelineId;
    @Schema(title = "编排版本号", required = true)
    private int resourceVersion;
    @Schema(title = "步骤ID", required = true)
    private String stageId;
    @Schema(title = "作业容器ID", required = true)
    private String containerId;
    @Schema(title = "执行次数", required = true)
    private int executeCount;
    @Schema(title = "执行变量", required = true)
    private Map<String, Object> containerVar;
    @Schema(title = "作业容器类型", required = true)
    private String containerType;
    @Schema(title = "构建状态", required = false)
    private String status;
    @Schema(title = "是否为构建矩阵组", required = false)
    private Boolean containPostTaskFlag;
    @Schema(title = "是否为构建矩阵组", required = false)
    private Boolean matrixGroupFlag;
    @Schema(title = "所在矩阵组ID", required = false)
    private String matrixGroupId;
    @Schema(title = "开始时间", required = true)
    private LocalDateTime startTime;
    @Schema(title = "结束时间", required = true)
    private LocalDateTime endTime;
    @Schema(title = "业务时间戳集合", required = true)
    private Map<BuildTimestampType, BuildRecordTimeStamp> timestamps;

    public BuildRecordContainer(String projectId, String pipelineId, int resourceVersion, String buildId, String stageId, String id, String classType, int executeCount, Boolean matrixGroupFlag, String s, Map<String, Object> containerVar, Map<BuildTimestampType, BuildRecordTimeStamp> kvHashMap) {
        this.buildId = buildId;
        this.projectId = projectId;
        this.pipelineId = pipelineId;
        this.resourceVersion = resourceVersion;
        this.stageId = stageId;
        this.containerId = id;
        this.containerType = classType;
        this.executeCount = executeCount;
        this.matrixGroupFlag = matrixGroupFlag;
        this.containerVar = containerVar;
        this.status = s;
        this.timestamps = kvHashMap;
        this.startTime = LocalDateTime.now();
        this.endTime = LocalDateTime.now();
    }

    public static class BuildRecordContainerHelper {
        @SuppressWarnings("ComplexMethod")
        public static void addRecords(
                List<BuildRecordContainer> buildRecordContainers,
                String stageId,
                boolean stageEnableFlag,
                Container container,
                StartBuildContext context,
                BuildStatus buildStatus,
                List<BuildRecordTask> taskBuildRecords
        ) {
            Map<String, Object> containerVar = new HashMap<>();
            containerVar.put(Container.class.getDeclaredFields()[0].getName(), container.getName());
            if (container.getContainerHashId() != null) {
                containerVar.put(Container.class.getDeclaredFields()[1].getName(), container.getContainerHashId());
            }
            Integer startVMTaskSeq = container.getStartVMTaskSeq();
            if (startVMTaskSeq != null) {
                containerVar.put(Container.class.getDeclaredFields()[2].getName(), startVMTaskSeq);
            }
            if (container instanceof TriggerContainer) {
                TriggerContainer triggerContainer = (TriggerContainer) container;
                containerVar.put(triggerContainer.getClass().getDeclaredFields()[0].getName(), triggerContainer.getParams());
                if (triggerContainer.getBuildNo() != null) {
                    containerVar.put(triggerContainer.getClass().getDeclaredFields()[1].getName(), triggerContainer.getBuildNo());
                }
                if (triggerContainer.getTemplateParams() != null) {
                    containerVar.put(triggerContainer.getClass().getDeclaredFields()[2].getName(), triggerContainer.getTemplateParams());
                }
            } else if (container instanceof VMBuildContainer) {
                VMBuildContainer vmBuildContainer = (VMBuildContainer) container;
                if (vmBuildContainer.getShowBuildResource() != null) {
                    containerVar.put(VMBuildContainer.class.getDeclaredFields()[0].getName(), vmBuildContainer.getShowBuildResource());
                }
            }
            buildRecordContainers.add(
                    new BuildRecordContainer(
                            context.getProjectId(),
                            context.getPipelineId(),
                            context.getResourceVersion(),
                            context.getBuildId(),
                            stageId,
                            container.getId(),
                            container.getClassType(),
                            context.getExecuteCount(),
                            container.getMatrixGroupFlag(),
                            buildStatus != null ? buildStatus.name() : null,
                            containerVar,
                            new HashMap<>()
                    )
            );
            if (taskBuildRecords == null || container.getMatrixGroupFlag()) {
                return;
            }
            for (int index = 0; index < container.getElements().size(); index++) {
                Element element = container.getElements().get(index);
                if (buildStatus == BuildStatus.SKIP && !ElementUtils.getTaskAddFlag(
                        element,
                        stageEnableFlag,
                        container.containerEnabled(),
                        container.getMatrixGroupFlag()
                )) {
                    continue;
                }
                int taskSeq = startVMTaskSeq != null && startVMTaskSeq > 1 && index < startVMTaskSeq - 1 ? index + 1 : index + 2;
                taskBuildRecords.add(
                        new BuildRecordTask(
                                context.getProjectId(),
                                context.getPipelineId(),
                                context.getBuildId(),
                                stageId,
                                container.getId(),
                                element.getId(),
                                element.getClassType(),
                                element.getAtomCode(),
                                context.getExecuteCount(),
                                context.getResourceVersion(),
                                taskSeq,
                                buildStatus != null ? buildStatus.name() : null,
                                element.initTaskVar(),
                                new HashMap<>(),
                                element.getAdditionalOptions() != null
                                        && element.getAdditionalOptions().getElementPostInfo() != null
                                        && !element.getAdditionalOptions().getElementPostInfo().getParentElementId().equals(element.getId())
                                        ? element.getAdditionalOptions().getElementPostInfo()
                                        : null
                        )
                );
            }
        }
    }
}
