package cd.casic.ci.process.engine.dispatcher.impl;

import cd.casic.ci.process.engine.dispatcher.BaseDispatcher;
import cd.casic.ci.process.engine.enums.ContextStateEnum;
import cd.casic.ci.process.engine.manager.RunContextManager;
import cd.casic.ci.process.engine.message.TaskRunMessage;
import cd.casic.ci.process.engine.runContext.SecondStageRunContext;
import cd.casic.ci.process.engine.runContext.TaskRunContext;
import cd.casic.ci.process.process.dataObject.base.PipBaseElement;
import cd.casic.ci.process.process.dataObject.pipeline.PipPipeline;
import cd.casic.ci.process.process.dataObject.stage.PipStage;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import cd.casic.framework.mq.redis.core.RedisMQTemplate;

import java.util.HashMap;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicInteger;

public class SerialDispatcher implements BaseDispatcher {
    private SecondStageRunContext stageRunContext;
    private List<PipTask> taskList;
    private CountDownLatch latch;
    private String triggerModel;
    private RedisMQTemplate redisMQTemplate;
    private RunContextManager contextManager;

    public SerialDispatcher(SecondStageRunContext stageRunContext, CountDownLatch latch,RunContextManager contextManager,RedisMQTemplate redisMQTemplate) {
        this.stageRunContext = stageRunContext;
        PipBaseElement contextDef = stageRunContext.getContextDef();
        if (contextDef instanceof PipStage secondStage) {
            this.triggerModel = secondStage.getTriggerMode();
            this.taskList = secondStage.getTaskValues();
        }
        this.latch = latch;
        this.redisMQTemplate = redisMQTemplate;
        this.contextManager = contextManager;
    }

    @Override
    public void dispatch() throws InterruptedException {
        for (PipTask pipTask : taskList) {
            // 注册taskContext，且发送消息至消息队列给work执行, 如果需要则传入参数
            TaskRunContext taskRunContext = new TaskRunContext(pipTask,stageRunContext,new HashMap<>());
            contextManager.contextRegister(taskRunContext);
            taskRunContext.changeContextState(ContextStateEnum.READY);
            TaskRunMessage taskRunMessage = new TaskRunMessage(pipTask);
            redisMQTemplate.send(taskRunMessage);
            // TODO 监听当前taskContext状态变成执行成功或者执行失败(worker当中改变状态为运行中、执行成功、执行失败)
            //
            AtomicInteger state = taskRunContext.getState();
            while (state.get() != ContextStateEnum.HAPPY_ENDING.getCode()
                    && state.get() != ContextStateEnum.BAD_ENDING.getCode()) {
                Thread.sleep(1000L);
            }
            //
        }
    }

    @Override
    public void run() {
        try {
            // TODO 检测触发方式如果需要手动触发，挂起当前stage，等待父级执行相应操作
            // TODO 看看需要内存入库还是忽略掉当前执行，进行入库（countDown放行）
//            stageRunContext.callParentChange();
            dispatch();
        } catch (Exception e) {
//            throw new RuntimeException(e);
            // TODO 当前stage标记为失败等待父context发现并处理
        }
        latch.countDown();
    }
}
