package cd.casic.pipeline.store.api.common;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.pipeline.store.pojo.common.comment.StoreCommentInfo;
import cd.casic.pipeline.store.pojo.common.comment.StoreCommentRequest;
import cd.casic.pipeline.store.pojo.common.comment.StoreCommentScoreInfo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;

@Tag(name = "USER_STORE_COMMENT", description = "研发商店-组件评论")
@RestController
@RequestMapping("/user/store/comments")
public interface UserStoreCommentResource {

    @Operation(summary = "获取组件评论接口")
    @GetMapping("/{commentId}/get")
    CommonResult<StoreCommentInfo> getStoreComment(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "评论ID", required = true)
            @PathVariable("commentId")
            String commentId
    );

    @Operation(summary = "获取组件的评论列表")
    @GetMapping("/types/{storeType}/codes/{storeCode}/list")
    PageResult<StoreCommentInfo> getStoreComments(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            String storeType,
            @Parameter(description = "组件代码", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "页码", required = true)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = true)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "获取组件的评分详情")
    @GetMapping("/types/{storeType}/codes/{storeCode}/score/get")
    CommonResult<StoreCommentScoreInfo> getStoreCommentScoreInfo(
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            String storeType,
            @Parameter(description = "组件代码", required = true)
            @PathVariable("storeCode")
            String storeCode
    );

    @Operation(summary = "新增组件评论")
    @PostMapping("/ids/{storeId}/codes/{storeCode}/add")
    CommonResult<StoreCommentInfo> addStoreComment(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件ID", required = true)
            @PathVariable("storeId")
            String storeId,
            @Parameter(description = "组件代码", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "评论信息请求报文体", required = true)
            StoreCommentRequest storeCommentRequest
    );

    @Operation(summary = "更新组件评论")
    @PutMapping("/{commentId}/update")
    CommonResult<Boolean> updateStoreComment(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "评论ID", required = true)
            @PathVariable("commentId")
            String commentId,
            @Parameter(description = "评论信息请求报文体", required = true)
            StoreCommentRequest storeCommentRequest
    );

    @Operation(summary = "评论点赞")
    @PutMapping("/{commentId}/praise")
    CommonResult<Integer> updateStoreCommentPraiseCount(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "评论ID", required = true)
            @PathVariable("commentId")
            String commentId
    );
}