package cd.casic.pipeline.store.api.common;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.common.enums.StoreTypeEnum;
import cd.casic.pipeline.store.pojo.common.member.StoreMemberItem;
import cd.casic.pipeline.store.pojo.common.member.StoreMemberReq;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;

@Tag(name = "USER_STORE_MEMBER", description = "STORE-组件-用户")
@RestController
@RequestMapping("/user/market/desk/store/member/")
public interface UserStoreMemberResource {

    @Operation(summary = "获取store组件成员列表")
    @GetMapping("/list")
    CommonResult<List<StoreMemberItem>> list(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "store组件标识", required = true)
            @RequestParam("storeCode")
            String storeCode,
            @Parameter(description = "store组件类型", required = true)
            @RequestParam("storeType")
            StoreTypeEnum storeType
    );

    @Operation(summary = "添加store组件成员")
    @PostMapping("/add")
    CommonResult<Boolean> add(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "添加成员请求报文")
            StoreMemberReq storeMemberReq
    );

    @Operation(summary = "删除store组件成员")
    @DeleteMapping("/delete")
    CommonResult<Boolean> delete(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "成员ID", required = true)
            @RequestParam("id")
            String id,
            @Parameter(description = "store组件标识", required = true)
            @RequestParam("storeCode")
            String storeCode,
            @Parameter(description = "store组件类型", required = true)
            @RequestParam("storeType")
            StoreTypeEnum storeType
    );

    @Operation(summary = "查看store组件成员信息")
    @GetMapping("/view")
    CommonResult<StoreMemberItem> view(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "store组件标识", required = true)
            @RequestParam("storeCode")
            String storeCode,
            @Parameter(description = "store组件类型", required = true)
            @RequestParam("storeType")
            StoreTypeEnum storeType
    );

    @Operation(summary = "修改store组件成员的调试项目")
    @PutMapping("/test/project/change")
    CommonResult<Boolean> changeMemberTestProjectCode(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "store组件成员", required = true)
            @RequestParam("storeMember")
            String storeMember,
            @Parameter(description = "项目代码", required = true)
            @RequestParam("projectCode")
            String projectCode,
            @Parameter(description = "store组件标识", required = true)
            @RequestParam("storeCode")
            String storeCode,
            @Parameter(description = "store组件类型", required = true)
            @RequestParam("storeType")
            StoreTypeEnum storeType
    );
}